 PRAGMA 6809,operandsizewarning
 OPT c
;**************************************************;
;            T E S T 5 1 2 K - M O                 ; 
;           (c) 2023 - Daniel Coulom               ;  
;           http://dcmoto.free.fr/                 ;
;--------------------------------------------------;
; Ce code est distribue gratuitement dans l'espoir ;
; qu'il sera utile, mais sans aucune  garantie  et ;
; sans  engager  la  responsabilit  de  l'auteur. ;
; Vous  pouvez  l' utiliser,  le  modifier  et  le ;
; diffuser librement, en conservant cette  licence ;
; et les rfrences de l'auteur dans   toutes  les ;
; copies. L'exploitation commerciale est interdite.;
;**************************************************;

; Ce programme teste l'extension memoire des MO.
; La taille (64K ou 512K) peut tre choisie.
; 1) Tester tous les bits (valeurs 0 et 1)
; 2) Ecrire des sequences de 255 octets diffrents 
; 3) Verifier que les memes sequences sont relues 

;**************************************************;
;                Version 2023.07.19                ;
;**************************************************;
; Historique
; 2023.07.19 Version compatible MO5, MO6, PC128
; 2021.01.16 Petite modification du  message de fin
; 2021.01.15 Correction anomalie affichage $A7CB
; 2021.01.14 Selection de 64K ou 512K
; 2021.01.13 Premiere version operationnelle

;---------------------------------
;Initialisations communes
;---------------------------------
  ORG   $9000
  PSHS  U,Y,X,DP,B,A,CC  ; empilage des registres                  
  ORCC  #$50           ; desactive les interruptions 
  CLRA                 ; A=0 (64K)
  STA   OPTION         ; option 64K par defaut
  LDA   $FFF0          ; type d'ordinateur
  BEQ   SEL            ; MO5, MO5E

;---------------------------------
;Initialisations MO6
;---------------------------------
  LDA   $2081          ; lecture du contenu de $A7E7
  ORA   #$40           ; set bit 6 (gestion par $A7CB)
  STA   $2081          ; sauvegarde du contenu de $A7E7
  STA   $A7E7          ; maj $A7E7
  LDA   #$0C           ; pour selection MEMO5 en ecriture 
  STA   $A7CB          ; maj $A7CB
  CLRA                 ; 0 pour maj $A7E4
  STA   $A7E4          ; clear bit 0 de $A7E4
  LDA   $A7E4          ; lecture D7-D4 de $A7DD
  ANDA  #$DF           ; raz bit 5 (cartouche visible)
  ORA   #$07           ; couleur de fond 7
  STA   $A7DD          ; maj $A7DD

;---------------------------------
;Selection de 64K ou 512K
;---------------------------------
SEL
  LDY    #TITRE        ; message de titre
  LBSR   WSTR          ; affichage message
  LDY    #T64K         ; message extension 64K
SEL1
  SWI                  ; appel fonction
  FCB    $0A           ; lecture clavier
  CMPB   #$0D          ; test'ENTREE'
  LBEQ   STOP          ; retour au BASIC 
  CMPB   #$30          ; test'0'
  BEQ    TESTB0        ; selection 64K 
  CMPB   #$31          ; test'1'
  BNE    SEL1          ; recommencer
  LDA    #$01          ; A=1 (512K)
  STA    OPTION        ; option 512K
  LDY    #T512K        ; message extension 512K

;---------------------------------
;Test de chaque bit
;---------------------------------
TESTB0
  LBSR   WSTR          ; affichage message
  LDY    #TESTB        ; message "Test des bits"
  LBSR   WSTR          ; affichage message
  LDA    #$0C          ; initialisation A 
TESTB1
  LDY    #POINT        ; point
  LBSR   WSTR          ; affichage message
  STA    $A7CB         ; selection de la banque
  BSR    TBITS         ; test des bits de la banque
  BNE    ERBITS        ; erreur detectee
  INCA                 ; banque suivante
  BITA   #$0F          ; test des quatre bits de poids faible
  BNE    TESTB1        ; passage a la banque suivante
  TST    OPTION        ; test de l'option 64K/512K
  BEQ    TESTB2        ; arret du test pour 64K
  ORA    #$0C          ; restauration des bits 2 et 3 
  BPL    TESTB1        ; passage a la banque suivante
TESTB2
  LDY    #NOERR        ; message "Aucune erreur"
  LBSR   WSTR          ; affichage message
  BRA    TESTA0        ; test des adresses

;---------------------------------
;Erreur detectee sur les bits
;---------------------------------
ERBITS
  LDY    #ERR1         ; message "Erreur sur les bits"
  LBSR   WSTR          ; affichage message
  BRA    RETOUR        ; abandon du programme
  
;---------------------------------------
;Test des bits d'une banque memoire
;---------------------------------------
TBITS
  LDX    #$B000        ; adresse de debut de la banque
  LDB    #$55          ; valeur a ecrire en RAM
TBIT1
  COMB                 ; inversion des bits
  STB    ,X            ; ecriture
  CMPB   ,X            ; lecture
  BNE    TBIT2         ; valeur lue incorrecte
  COMB                 ; inversion des bits
  STB    ,X            ; ecriture
  CMPB   ,X            ; lecture
  BNE    TBIT2         ; valeur lue incorrecte
  CMPX   #$EFFF        ; adresse de fin de la banque
  BEQ    TBIT2         ; fin du controle
  LEAX   1,X           ; adresse suivante
  BRA    TBIT1         ; controle adresse suivante
TBIT2
  RTS       

;---------------------------------
;Test de toutes les adresses
;---------------------------------
TESTA0
  LDD    #$0C00        ; initialisation A et B 
  LDY    #WSEQU        ; message "Ecriture sequence"
  BSR    WSTR          ; affichage message
TESTA1
  LDY    #POINT        ; point
  BSR    WSTR          ; affichage message
  STA    $A7CB         ; selection de la banque
  BSR    WRITE         ; ecriture des sequences dans la banque
  INCA                 ; banque suivante
  BITA   #$0F          ; test des quatre bits de poids faible
  BNE    TESTA1        ; passage a la banque suivante
  TST    OPTION        ; test de l'option 64K/512K
  BEQ    TESTA2        ; arret du test pour 64K
  ORA    #$0C          ; restauration des bits 2 et 3 
  BPL    TESTA1        ; passage a la banque suivante
TESTA2
  LDD    #$0C00        ; initialisation A et B 
  LDY    #RSEQU        ; message "Lecture sequence"
  BSR    WSTR          ; affichage message
TESTA3
  LDY    #POINT        ; point
  BSR    WSTR          ; affichage message
  STA    $A7CB         ; selection de la banque
  BSR    CTRL          ; controle des sequences dans la banque
  BNE    ERADD         ; erreur detectee
  INCA                 ; banque suivante
  BITA   #$0F          ; test des quatre bits de poids faible
  BNE    TESTA3        ; passage a la banque suivante
  TST    OPTION        ; test de l'option 64K/512K
  BEQ    TESTA4        ; arret du test pour 64K
  ORA    #$0C          ; restauration des bits 2 et 3 
  BPL    TESTA3        ; passage a la banque suivante
  LDA    #$80          ; pour obtenir $7F apres DECA     
TESTA4
  DECA                 ; derniere valeur $A7CB
  LDY    #NOERR        ; message "Aucune erreur"
  BSR    WSTR          ; affichage message
  BRA    RETOUR        ; retour         

;---------------------------------
;Erreur adresses
;---------------------------------
ERADD
  LDY    #ERR2         ; message "Erreur adresses"
  BSR    WSTR          ; affichage message

;---------------------------------
;Retour au BASIC
;---------------------------------
RETOUR 
  LDY    #BANK         ; message "Fin du test - $A7CB="
  BSR    WSTR          ; affichage message
  BSR    HEXVA         ; affichage valeur hexa de A
  LDY    #ADDRESS      ; message "  Adresse="
  BSR    WSTR          ; affichage message
  BSR    HEXVX         ; affichage valeur hexa de X
STOP
  LDY    #SAUT         ; saut d'une ligne
  BSR    WSTR          ; affichage message
  CLR    $A7CB         ; deselection MEMO5
  PULS   CC,A,B,DP,X,Y,U,PC ; retour

;------------------------------------------------------
; AFFICHAGE CHAINE TERMINEE PAR ZERO (Adresse dans Y)
;------------------------------------------------------
WSTR
  PSHS  X,B,A          ; sauvegarde registres
WSTR1  
  LDB   ,Y+            ; caractere a ecrire
  BEQ   WSTR9          ; fin de la chainee
  SWI                  ; appel fonction
  FCB   $02            ; $02 = envoi caractere
  BRA   WSTR1          ; caractere suivant
WSTR9
  PULS  A,B,X,PC       ; retour

;-----------------------------------------------
;Ecriture des sequences dans une banque memoire
;-----------------------------------------------
WRITE
  LDX    #$B000        ; adresse de debut de la banque
WRITE1
  INCB                 ; valeur suivante de $01 a $FE 
  CMPB   #$FF          ; fin de sequence
  BNE    WRITE2        ; ecrire la valeur
  CLRB                 ; recommencer a zero
WRITE2
  STB    ,X+           ; ecriture
  CMPX   #$F000        ; adresse de fin de la banque
  BNE    WRITE1        ; ecriture adresse suivante
WRITE3
  RTS

;-----------------------------------------------
;Controle des sequences dans une banque memoire
;-----------------------------------------------
CTRL
  LDX    #$B000        ; adresse de debut de la banque
CTRL1
  INCB                 ; valeur suivante de $01 a $FE 
  CMPB   #$FF          ; fin de sequence
  BNE    CTRL2         ; controler la valeur
  CLRB                 ; recommencer a zero
CTRL2
  CMPB   ,X            ; comparaison
  BNE    CTRL3         ; valeur lue incorrecte
  CMPX   #$EFFF        ; adresse de fin de la banque
  BEQ    CTRL3         ; controle adresse suivante
  LEAX   1,X           ; adresse suivante
  BRA    CTRL1         ; controle adresse suivante
CTRL3
  RTS       

;------------------------------------------------------
; Affichage valeur hexadecimale
;------------------------------------------------------
DISPLAYHEXA
  LDY   #HEXA          ; valeur hexa
  BSR    WSTR          ; affichage valeur hexa
  RTS                  ; retour

;------------------------------------------------------
; CALCUL VALEUR HEXADECIMALE DE X
;------------------------------------------------------
HEXVX
  TFR   X,D            ; transfert dans D   
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  BSR   HCAR           ; calcul caractere
  STA   HEXA+1         ; stockage premier caractere
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  LSRB                 ; decalage droite
  TFR   B,A            ; transfert dans A
  BSR   HCAR           ; calcul caractere
  STA   HEXA+3         ; stockage troisieme caractere
  TFR   X,D            ; transfert dans D   
  ANDA  #$0F           ; isole 4 bits poids faible   
  ANDB  #$0F           ; isole 4 bits poids faible   
  BSR   HCAR           ; calcul caractere
  STA   HEXA+2         ; stockage deuxieme caractere
  TFR   B,A            ; transfert dans A
  BSR   HCAR           ; calcul caractere
  STA   HEXA+4         ; stockage quatrieme caractere
  BRA   DISPLAYHEXA    ; affichage de la valeur hexadecimale

;------------------------------------------------------
; CALCUL VALEUR HEXADECIMALE DE A
;------------------------------------------------------
HEXVA
  PSHS  A              ; sauvegarde de A              
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  LSRA                 ; decalage droite
  BSR   HCAR           ; calcul caractere
  STA   HEXA+1         ; stockage premier caractere
  PULS  A              ; restauration de A
  ANDA  #$0F           ; isole 4 bits poids faible   
  BSR   HCAR           ; calcul caractere
  STA   HEXA+2         ; stockage deuxieme caractere
  LDA   #$20           ; espace
  STA   HEXA+3         ; stockage troisieme caractere
  STA   HEXA+4         ; stockage quatrieme caractere
  BRA   DISPLAYHEXA    ; affichage de la valeur hexadecimale

;------------------------------------------------------
; CALCUL CARACTERE HEXADECIMAL (A)
;------------------------------------------------------
HCAR
  CMPA  #$0A           ; test lettre
  BLT   HCAR1          ; chiffre
  ADDA  #$07           ; ajout de 7 pour les lettres
HCAR1
  ADDA  #$30           ; ajout de $30 pour valeur ASCII
  RTS                  ; retour
          
;------------------------------------------------------
; Messages a afficher
;------------------------------------------------------
TITRE
  FCB   $0C            ; effacement ecran   
  FCB   $1B,$20,$40    ; couleur noir
  FCB   $1B,$20,$57    ; fond blanc
  FCB   $1B,$67        ; bordure blanche
  FCC   " ------------------------------"
  FCB   $0D,$0A
  FCC   "| Test extension memoire MEMO5 |"
  FCB   $0D,$0A
  FCC   " ------------------------------"
  FCB   $0D,$0A,$0A
  FCC   "Tapez 0 (64K) ou 1 (512K)"
  FCB   $0D,$0A,$0A
  FCC   "ou ENTREE pour abandonner"
  FCB   $00
T64K
  FCB   $0D,$0B,$0B
  FCC   "Test de l'extension 64K  "
  FCB   $00
T512K
  FCB   $0D,$0B,$0B
  FCC   "Test de l'extension 512K "
  FCB   $00
TESTB
  FCB   $0D,$0A,$0A
  FCC   "1) Test de chaque bit (0 et 1)"
  FCB   $0D,$0A,$00
WSEQU
  FCB   $0D,$0A,$0A
  FCC   "2) Ecriture sequence pour test adresses"
  FCB   $0D,$0A,$00
RSEQU
  FCB   $0D,$0A,$0A,$0A
  FCC   "3) Verification de la sequence ecrite"
  FCB   $0D,$0A,$00
ERR1  
  FCB   $0D,$0A,$0A
  FCC   "ERREUR DETECTEE SUR LES BITS"
  FCB   $0D,$0A,$00
ERR2  
  FCB   $0D,$0A
  FCC   "ERREUR DETECTEE SUR LES ADRESSES"
  FCB   $0D,$0A,$00
NOERR  
  FCB   $0D,$0A
  FCC   "AUCUNE ERREUR DETECTEE"
  FCB   $0D,$0A,$00
BANK
  FCB   $0D,$0A,$0A
  FCC   "Fin du test  $A7CB="
  FCB   $00
ADDRESS
  FCC   "Adresse="
  FCB   $00
SAUT  
  FCB   $0D,$0A,$0A,$00
HEXA
  FCB   $24,$30,$30,$30,$30,$00
POINT
  FCB   $2E,$00  
OPTION
  FCB   $00